\documentclass{report}
\usepackage{RCS}
\RCS$Revision: 1.3 $
\RCS$Date: 2001/04/07 21:15:38 $
\usepackage{hyperref}
\usepackage{acronym}
\usepackage{moreverb}
\renewcommand{\Diamond}{\relax}
\newcommand{\app}[1]{\textsc{#1}\index{#1}}
\newcommand{\tag}[1]{\texttt{<#1>}}
\newcommand{\file}[1]{\texttt{#1}}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\title{DocBook-Based Literate Programming\\
       {\small\RCSRevision}}
\author{Mark Wroth}
\date{\RCSDate}
\begin{document}\maketitle

This document, and the literate programming system described herein,
is copyright \copyright\ 2001 by Mark Wroth. Permission is granted for 
the reproduction, use, and modification of this system provided that 
\begin{itemize}
\item No charge is made for the use of this system. A nominal charge
  for reproduction of the media on which the system is provided may be
  levied, however.
\item The complete literate programming source of the system is made
  available to the user.
\item Modifications of this system are clearly identified as
  derivative works.  Provision must be made to identify the author of
  the changes, and to provide error reports to the modifier rather
  than the original author.
\end{itemize}

Works created using this system (as opposed to modifications of this
system itself) are subject to whatever copyright or use provisions may
be imposed by the authors of those works; the use of this system shall
not be the basis for any change or modification of the rights of such
authors. 

The author of this system, in permitting its general use, makes no
warranty of its suitability for any purpose nor any guarantee of its
correctness. In other words, you are welcome to use this system, but
you do so at your own risk.

\clearpage
\tableofcontents

\chapter{Functional Description}

The DocBook-based Literate Programming system provides a mechanism to
write literate programs using a minor extension of the \ac{SGML}
\app{DocBook} \ac{DTD}.

The system consists of two main parts:
\begin{itemize}
\item A \ac{DTD} that extends \app{DocBook} to add the logic needed
  for literate programming.  These are relatively minor extensions to
  the basic DTD.  The details are discussed in Chapter~\ref{sec:DTD}.
\item \ac{DSSSL} style sheets that, together with a \ac{DSSSL} engine
  that implements some of James Clark's extensions, serve as ``weave''
  and ``tangle'' processors.  These style sheets are discussed in
  Chapters~\ref{sec:weave} and \ref{sec:tangle}, respectively.
\end{itemize}

This document also discusses the design considerations behind the
implementation, and provides a short sample document that serves as an
example of how the \ac{DTD} is used (and serves as a simple test case).

\section{Purpose of the Functional Description}
\label{sec:purpose}

This functional description for ``DocBook-based Literate Programming''
is written to provide:
\begin{enumerate}
\item The system requirements to be satisfied which will serve as a
  basis for the system design.
\item Information on performance requirements, preliminary design
  considerations, and user impacts including fixed and continuing
  costs.
\item A basis for development of system tests.
\end{enumerate}

\section{Project References}
\label{sec:references}

Documents significant to this project are discussed in the bibliography.

\chapter{System Summary}
\label{sec:summary}


\section{Background}
\label{sec:background}

%This paragraph shall present background information concerning the
%uses and purposes of the system.  Reference must be made to
%interfacing systems when needed to enhance the general description.
%The relationships between the project and other capabilities being
%developed concurrently shall be described.


\subsection{Literate Programming}
\label{sec:lp}

\textit{Literate programming} is a style of computer programming
pioneered by Professor Donald Knuth in the early 1980's (the defining
paper was published in \textsl{The Computer Journal} in May 1984, at
which point the earliest literate programming system, \app{WEB}, was
already functional).  

The key tenent of literate programming is that computer programs are
written to be read and understood by \emph{human beings} as well as
computers---and the organization of the program's source code should
allow the program author to explain the purpose and implementation of
the code to the human audience.  In Knuth's own words, ``Instead of
imagining that our main task is to instruct a \emph{computer} what to
do, let us concentrate rather on explaining to \emph{human beings}
what we want a computer to do.''\footnote{``Literate Programming,''
  originally published in \textsl{The Computer Journal}, May 1984,
  quoted in \cite[p.~99]{Knuth92}}

To this end, a literate programming system typically supports several
functions:
\begin{itemize}
\item Mechanisms to extract or otherwise make available to the
  computer the code instructions making up the computer program in a
  form usable by the computer (e.g. in a form suitable for submission
  to the compiler), and to translate the source code and documentation
  into appropriately rendered documentation.  
\item The ability to write code documentation using the full range of
  typographic features used in normal typesetting.
\item The ability to arbitrarily intermingle code and documentation.
\item The ability to arbitrarily reorder code fragments so that the
  exposition of the program design may precede in an order appropriate
  for the human reader, while retaining the instruction order required
  for the computer to correctly execute the program.
\end{itemize}
The process of making the code instructions available to the computer
is called the ``tangle'' phase of processing, while the process of
rendering the documentation is called the ``weave'' phase.  These
names derive from the names of the two programs making up the original
\app{WEB} system that performed these functions.

Some literate programming processors, including Knuth's original
\app{WEB} system, additionally ``pretty print'' the programming
language instructions (for example, by setting the reserved words of
the language in boldface type).  This pretty print capability is not
universal, and in fact is not desired by some programmers. Another
capability sometimes found is the ability to define macros in the
literate programming system, usually to supplement the capabilities of
the underlying programming language.

Since the introduction of Knuth's \app{WEB} system, which was used for
the \app{\TeX} and \app{MetaFONT} programs, a variety of other
literate programming systems have appeared, as have a number of other
mechanisms for improved commenting of source code\footnote{The
  boundary between ``literate programming'' systems and ``improved
  commenting'' mechanisms is somewhat subjective.  However, for the
  purposes of this discussion, a system is considered a literate
  programming system if it offers the capabilities listed
  above.}. 
A complete review of the available systems is beyond
the scope of this discussion. However, experience with the original \app{WEB}
system, John Krommes' \app{FWEB}, and Preston Brigg's \app{Nuweb}
literate programming systems, and the \app{doc} \LaTeXe\ documentation
system and Normal Walsh's DocBook-based \ac{DSSSL} style sheet
documentation systems (both of which may be characterized as improved
commenting systems) has been a significant factor in defining
the characteristics desired in this system.

\subsection{SGML and XML}
\label{sec:sgml}

The \acl{SGML} is defined in \ac{ISO} Standard ISO 8879:1986, and
defines a mechanism for \emph{defining} markup languages and enforcing
certain relationships among the data contained in appropriately marked
up documents.  Among other things, \ac{SGML} provides a clear
mechanism for making explicit what parts of a document have what role,
and does so in a way that encourages the construction of tools able to
parse such documents---and hopefully do useful things with the parsed
information.

In the late 1990's \ac{SGML} was supplemented by \ac{XML}, which is a
simplified \ac{SGML} application designed to make it easier to
construct parsers and other processing tools.  In general, \ac{XML}
has the same basic functionality as \ac{SGML}, with some of the
lesser-used features of the language omitted. Tools that
can process \ac{SGML} documents can also usually process \ac{XML}
documents; it is not generally the case than an \ac{XML} tool can
process a general \ac{SGML} document.

\subsection{DocBook}

One of the useful features of \ac{SGML} is the ability to create
\ac{DTD}s that describe the structure of documents and the markup that
makes that structure explicit.  This permits the creation of general
document types---and the tools to process them.

One such document type is \app{DocBook}, a document type created for
computer-oriented technical books.  \app{DocBook} is maintained by
\ac{OASIS}, and has evolved into a flexible and robust document
definition, supported by a variety of tools.  \app{DocBook} exists in
both \ac{SGML} and \ac{XML} versions.

In particular, Norman Walsh has defined a set of \ac{DSSSL} style
sheets that process \app{DocBook} documents and produce printed (or
\ac{HTML}) output renderings. These style sheets are both extensible
and customizable, and serve as a significant base for
computer-oriented documentation---such as a literate program.

\section{Objectives}
\label{sec:objectives}

%  This paragraph shall state the major          
%   performance requirements and goals of the proposed system.     
%   These statements should be concise, quantified if possible,    
%   and may include examples.  When applicable, related events,    
%   such as exercises or impending military operations, may be     
%   discussed.

This project creates a set of extensions to the \app{DocBook}
\ac{SGML} \ac{DTD} to allow its use for literate programming markup.
The resulting system shall 
\begin{itemize}
\item Provide a mechanism to extract program files from the literate
  programming source in appropriate forms for their use as source code
  in the intended programming language or languages.
\item Permit the use of existing \app{DocBook}-based tools with only
  minor modifications (ideally none) to produce documentation of
  software projects.
\end{itemize}


\section{Existing Methods and Procedures}
\label{sec:existing}

%This paragraph shall briefly describe the current methods and
%procedures being employed to satisfy the existing information
%requirements.  A chart must be provided depicting the existing data
%flow through the functional system from data acquisition through its
%processing and eventual output.  This chart may be complemented by an
%explanation or another chart showing the sequence in which the
%operational functions are performed by the user and identifying the
%support provided by the present system for decision making.
%Additionally, at least the following information should be included in
%this description:
%
%        a.  Organizational and personnel responsibilities.
%
%        b.  Equipment being used.
%
%        c.  Inputs and outputs including volume and frequency.
%
%        d.  Provisions in the system design for operation in
%        degraded modes or at alternate sites in cases of
%        emergency, disaster, or accident.
%
%        e.  Deficiencies, including limitations, such as time
%        delays.

There are a variety of literate programming systems in use at the
current time.  In general, they fall into three main categories:
\begin{itemize}
\item Language-aware systems.  These systems are designed to support a
  single computer programming language, and are marked by the ability
  to do limited parsing of code sections, usually accompanied by
  ``pretty printing'' of the computer source code.  Knuth's original
  \app{WEB} system falls into this category.  Most language aware
  systems use \TeX\ as their typesetting system.
\item Language-independent systems.  These systems attempt no parsing
  of the code sections. Most language-independent systems use \TeX\ as
  their typesetting systems, although there is some move towards
  \ac{HTML} as a documentation language.
\item Comment-based systems.  These systems extend the comment
  structures of the supported language in an attempt to provide usable
  documentation. Examples of such systems are the ``doc'' system used
  to document \LaTeXe\ packages, and---I believe---the ``Javadoc''
  system. 
\end{itemize}

Most or all of these existing systems target a specific output format,
usually the printed page, rendered via the \TeX\ typesetting system.
In part, this is a historical accident; the first literate programming
system used \TeX.  However, attempts to implement other documentation
languages (notably an attempt to write a `C' language literate
programming system with \app{troff} as the documentation processor)
indicate that the demands on the documentation branch of a literate
programming system are relatively taxing. This has undoubtedly
contributed to the limited number of output forms supported, although
some attempts to support \ac{HTML} have been made.

However, I am not aware of any literate programming systems based on
\ac{SGML} markup of the source code.  This omission seems unfortunate,
given the obvious applicability of \ac{SGML} markup to the process of
defining a literate program.

\section{Proposed Methods and Procedures}
\label{sec:proposed}

%This paragraph shall describe proposed methods and procedures.  This
%description, written in noncomputer-oriented language, shall explain
%how the proposed system will interact with the functional processes
%which the automated system will support.  Products from other systems
%that will be used with or become part of the proposed system shall be
%Identified.

%A chart depicting the proposed data flow should be provided to
%present an overall view of the planned capabilities.  If the
%proposed system will eliminate or degrade any capabilities in the
%existing system, these capabilities must also be identified and
%reasons stated for their elimination or degradation.  Alternative
%methods and procedures that have been considered may be included. 
%A chart showing the major functional processing steps and a chart
%showing the interacting organizations should be included within
%the following paragraphs wherever they best complement the
%narrative.

This project implements an \ac{SGML} markup-based literate programming
system.  Literate computer programs are written using some form of
text editor---preferably, but not necessarily an \ac{SGML}-aware
editor.  The documentation and programming language code is marked up
using an extension of the \app{DocBook} \ac{DTD}.  Once the literate
program is written (partially or completely), it is processed using a
variant of the \app{Jade} \ac{DSSSL} engine with either a ``tangle''
style-sheet (which is a stand-alone style sheet provided by this
project), or a ``weave'' style sheet (which extends the \app{DocBook
  Modular Style Sheets} in several small but important ways).

\subsection{Summary of Improvements}
\label{sec:improvements}

%This paragraph shall provide a qualitative and quantitative summary of
%the benefits to be obtained from the proposed system.  Required
%capabilities that will be satisfied by the proposed system shall be
%explicitly identified.  A comparison of the deficiencies identified in
%paragraph 2.3 and the identification of any additional capabilities
%required, along with appropriate explanations, may be provided.  When
%improvements of the existing methods and procedures are a requirement,
%the extent of the anticipated improvements must be stated.  The
%discussion may include:
%
%        a.  Functional Improvements (new capabilities).
%
%        b.  Improvements of degree (upgrading existing            
%        capabilities).
%
%        c.  Timeliness (improved response time).

Creating an \ac{SGML}-based literate programming system makes it
possible to exploit the wide variety of \ac{SGML} (or, with minor
variations, \ac{XML}) tools.  In particular, this makes it
straightforward to produce different output formats, such as a printed
version or an \ac{HTML} version.

The use of \ac{SGML} also separates the definition of the document
markup from the definition of the processing tools.  In principle,
this allows many different tools to be used with literate programs
written with this system. This advantage is largely theoretical at
this point, however.

\subsection{Summary of Impacts}
\label{sec:impacts}

%This and the following paragraphs shall describe the anticipated
%impacts and associated costs of the proposed system on the existing
%organizational and operational environments of the user.  Impacts on
%the user during the development of the system shall be noted.
 
\subsection{Assumptions and Constraints}
\label{sec:assumptions}

%This paragraph shall describe any user assumptions and constraints
%that will affect development and operation of the system.  Any
%limitations affecting the desired capability and explicit
%identification of any desired capabilities that will not be provided
%by the proposed system shall be discussed.  Any anticipated
%operational changes that will affect the proposed operation of the
%system shall be discussed.

It is assumed that the user of this system is familiar with the use of
\ac{SGML}-based tools, and the \app{DocBook} \ac{DTD}.

The processing \ac{DSSSL} style sheets assume the presence of selected
\ac{DSSSL} extensions implmented in James Clark's \app{Jade} engine
(specifically the ``entity'' and ``processing-instruction'' flow
objects). This capability is necessary to use the specific style
sheets provided by this project, but the use of the \ac{DTD} is
not affected.

\section{Detailed Characteristics}
\label{sec:details}


\section{Design Considerations}
\label{sec:design}

\begin{itemize}
\item Minimize changes to the DocBook DTD to allow processing using existing
output tools.

\item Keep the existing \tag{programlisting} as the basis for a scrap
  (again to minimize changes needed in the output processors).
  
\item Use added attributes for file output, definition and reference to
  continued and continuation scraps.  Use of both continued and
  continuation markup is semantically redundant, but will
  make processing of the ``weave'' branch easier---I think.
  
\item Use \tag{xref} for reference to definition scraps, and the
  \texttt{xreflabel} attribute for the definition scrap title. Again,
  this is driven by a desire to minimize changes that would impact the
  output processing tools.

\item Make maintaining,  modifying, and adding this functionality to other
  document types as easy as possible (see Section~\ref{sec:flexibility}).
\end{itemize}

\subsection{System Description}
\label{sec:system-description}

%This paragraph shall provide a general description of the design of
%the proposed AIS.  Related and interfacing systems and their
%documentation will be referred to as required to enhance this general
%description.  This description shall include a chart showing the
%relationship of the user organizations to the major components of the
%proposed AIS.  This chart shall be based on the information included
%in paragraph 2.4.


\subsection{System Functions}
\label{sec:system-functions}

%This paragraph shall describe the functions of the proposed AIS.  The
%performance requirements stated in paragraph 3.1 and the functions
%described by paragraph 3.2 must be elaborated upon here in enough
%detail that the system environment in Section 5 can be related to
%them.

This system performs three basic functions:
\begin{itemize}
\item Provide a \ac{DTD} that allows the markup of literate programs,
  including a flexible system for describing the purpose and
  implementation of the computer program (based on \app{DocBook}) and
  markup of the program code itself to allow the literate program to
  produce the computer instructions.
\item A \app{tangle} mechanism that actually produces the computer
  instructions from the literate programming source code.  This
  implementation, \texttt{SGMLTangle.dsl} is a \ac{DSSSL} style sheet
  using extensions to the \ac{DSSSL} standard as implemented in James
  Clark's \app{Jade} \ac{DSSSL} engine.\index{DSSSL extensions}
\item A  \app{weave} implementation that renders the literate
  programming source into useful documentation.  This style
  specification, \texttt{SGMLWeave.dsl}, extends Norman Walsh's
  \textsl{Modular DocBook Style Sheets}.  It provides both print and
  \ac{HTML} output, in the style sheets \texttt{print} and
  \texttt{html} respectively. 
\end{itemize}

\subsection{Flexibility}
\label{sec:flexibility}
%This paragraph shall describe the capability to be incorporated for
%adapting the system to changing requirements such as anticipated
%operational changes, interaction with new or improved systems, and
%planned periodic changes.
It is the intention of this system to:
\begin{itemize}
\item Maintain the ability to update the extensions to new versions of
\app{DocBook} as they are published.

\item Make the extensions as easy as possible to to move between the
\ac{SGML} and \ac{XML} versions.

\item Make it as simple as possible to add the literate programming
functionality to other \app{DocBook}-based \ac{DTD}s.

\item Provide a basis on which other implementations of the \app{tangle} and
\app{weave} functions could be built to support other tool chains.
\end{itemize}

%\subsection{System Data}
%\label{sec:data}


\section{Environment}
\label{sec:environment}

%This section shall describe the current ADP environment and
%project the environment needed to satisfy those requirements
%delineated in Sections 2 and 3.

   
\subsection{Equipment Environment}
\label{sec:equipment}
   
%This paragraph shall describe the equipment capabilities required for
%the operation of the proposed AIS.  This paragraph will present broad
%descriptions of the equipment presently available and the
%characteristics of any new equipment necessary; based on the
%information in Section 3.  A guideline for equipment to be described
%follows:
%
%        a.  Processors, including number of each online/offline, 
%        and size of internal storage.
%
%        b.  Storage media, including number of disk units, tape 
%        units, etc.
%
%        c.  Output devices, including number of each online/      
%        offline.
%
%        d.  Input devices, including number of each online/       
%        offline.

The hardware required to use this system is defined by the selected
software tools (in particular the \ac{SGML} processor).  Development
and testing was accomplished on 32-bit \app{Windows}-based computers.

\subsection{Support Software Environment}
\label{sec:supportsoftware}

%This paragraph shall describe the support software with which the
%applications software to be developed must interact. Included will be
%support software, input and equipment simulators, and test software,
%if needed.  The correct nomenclature, level (version), and
%documentation references of each such software system, subsystem, and
%software unit shall be provided.  In addition, the language, the
%operating system, and any Database Management System to be used will
%be identified.

Effective use of this system requires three major classes of
supporting software.  Except as noted with regard to the \app{tangle}
application, it is not necessary that the actual supporting software
used in the development of this system be available.


\subsubsection{Programming Editor}

Some form of text editor is needed to write the literate program.  The
minimum necessary functionality is the ability to write a plain text
output file.% 
\footnote{Actually, even this is an overstatement: the editor has to
  be able to produce \ac{SGML} input files compatible with the
  \ac{SGML} and \ac{DSSSL} processors being used. While this is
  \emph{usually} a plain text file, there are other ways to implment
  \ac{SGML} systems.}

The authoring process will be much easier, however, if the editor
supports both \ac{SGML} markup and the programming language or
languages in which the code is being written. A customizable editor
such as \app{emacs} is probably a useful choice.


\subsubsection{SGML Processors}

The literate programming \ac{DTD} extends the \app{DocBook} \ac{DTD},
and therefore requires the underlying \ac{DTD}.  This implementation
specifically uses the \ac{SGML} \ac{DTD}, Version 4.1 as the basis for
its extension.

To use this system for literate programming without additional
customization, a \ac{DSSSL} engine that implements James Clark's
\texttt{entity} and \texttt{processing-instruction} extensions to the
\ac{DSSSL} standard is needed. The \app{Jade} or \app{OpenJade}
engines meet this requirement.

The \app{weave} style sheet is based on Norman Walsh's \textsl{Modular
  DocBook Style Sheets} \cite{walsh-dsssl}, which must be available if
the \texttt{SGMLWeave.dsl} style specification is used.

As an \ac{SGML} application, of course, documents marked up with this
\ac{DTD} can, in principle, be used by any \ac{SGML}-compliant tool.

While this \ac{DTD} and the associated \ac{DSSSL} style sheets were
written for the \ac{SGML} version of the \app{DocBook} \ac{DTD}, it
should require only minor changes to re-implement this system in
\ac{XML}. This has not, however, been tested.


%\section{Security}
%\label{sec:security}

\section{System Development Plan}

%This section shall discuss the overall management approach to the
%development and Implementation of the proposed computer system. 
%Included shall be a discussion of and schedule for the
%documentation to be produced, time frames for the development of
%the system or the modules of the system, necessary liaison and
%participation by other organizations to ensure successful
%development, and any other factors that must be known prior to
%initiating development.  Reference may be made to such
%information contained in other documents.

This system has been implemented using the \app{Nuweb} literate
programming processor, which is a \TeX-based system.  The choice to
implement the system in this way was a ``bootstrapping'' choice; until
this system achieved basic functionality, it would be difficult to
implement the system in an \ac{SGML}-based system.

Because the original implementation was written using \app{Nuweb},
basic maintenance of the system is currently intended to continue in
that system.  Eventually, the system may be translated into itself,
but this is not currently planned.

Once basic functionality has been demonstrated, this system will be
made publically available by publication on the World-Wide Web. Future
modifications and extensions may or may not be made, depending on the
author's use of the system and feedback from other users (if any).

%\section{Cost Factors}

%This section shall provide a summary of cost factors for the
%proposed system In accordance with DoD Instruction 7041.3, when
%applicable.  While the proposed system responds directly to the
%project request or other initiation document, other factors may
%determine the need for this system, such as requirements of
%higher echelons of command, security considerations,
%telecommunications considerations, and the need to interface with
%other automated systems.  General alternatives that may be
%discussed include those for system development and system design
%with consideration being given to equipment, software, supporting
%telecommunications requirements, organization, operation, etc. 
%Reference may be made to such information contained in other
%documents. 


\chapter{DTD Implementation}
\label{sec:DTD}

\section{Purpose}


\section{Top Level Organization}

@O dblp.dtd @{<!--
DBLP.DTD; a literate programming DTD based on DocBook
PUBLIC 
"-//Mark Wroth//DTD DocBook V4.1-Based Extension Literate Programming 1.0//EN"
-->
@<Add \tag{programlisting} attributes@>
<!ENTITY % programlisting.element "IGNORE">
<!ENTITY % docbook PUBLIC "-//OASIS//DTD DocBook V4.1//EN">
%docbook;
@<Redefine the \tag{programlisting} element@> 
@<The `literalchar' element@>
@}


\section{The programlisting Customization}


We will set up the \tag{programlisting} element to ignore the
\app{DocBook} defined definition and substitute our own.  
@D Redefine the \tag{programlisting} element @{<!ELEMENT programlisting - - 
  ((CO | LineAnnotation | literalchar | %para.char.mix;)+)>
@}
To the attribute list, we add the attributes that we will use for
literate programming:
\begin{description}
\item[file] The file name the code is to be written to. This attribute
  is required for the scraps which begin output files.
\item[continuedfrom] The ID of the scrap this scrap continues.
\item[continuedin] The ID of the scrap this file continues.
\end{description}
We also make use of several of the attributes already defined for
programlisting: 
\begin{description}
\item[ID] Unique identifier for this scrap; required for scraps which
  are either continued or continue others.
\item[xreflabel] The title of the scrap; used for the head of a
  definition scrap.
\end{description}
@D Add \tag{programlisting} attributes @{<!ENTITY % local.programlisting.attrib "
 file          CDATA  #IMPLIED -- file name for output file --
 continuedfrom IDREF  #IMPLIED
 continuedin   IDREF  #IMPLIED "> @}

The choice to use both a \texttt{continuedfrom} and a
\texttt{continuedin} attribute allows the creation of a doubly-linked
list for each code section.  This permits the programmer to order the
code scraps in any desired sequence, while permitting the processing
system to easily traverse the scraps that make up the section.

While one or the other direction of the links between scraps is
sematically redundant in an absolute sense (with adequate effort, the
backward links could be constructed from the forward set, and
vice-versa), including both sets of links makes construction of the
processing tools much simpler. 

\subsection{The `literalchar' element} 
 
The following definitions are used to provide a workaround to get an 
actual ``less than'' character into the \ac{SGML} output. Since the 
character has syntactic meaning to the \ac{SGML} parser, by default  
it is `escaped' when placed in the \ac{SGML} output as character data. 
 
By defining an element to contain the required information, we let the
\ac{DSSSL} processor have access to it.  Defining entity references to
it simplifies the actual data entry.  If particular combinations seem
appropriate for a specific programming language it would make sense to
define entities which make syntactic sense.  This would allow one to
use, for example \texttt{\&logicaland;} instead of
\texttt{\&amp;\&amp;}\footnote{The basic suggestion to use a
  formatting-instruction to address the problem came from David
  Carlisle \texttt{davidc@@nag.co.uk} in a post to the DSSSList,
  Vol~3, Number~241, although the actual implementation does not
  follow his suggestions precisely.}.
 
@D The `literalchar' element @{ 
<!ELEMENT literalchar  - o EMPTY  
       -- literal data, to be handled in the DSSSL --> 
<!ATTLIST literalchar data CDATA #REQUIRED> 
<!ENTITY  lessthan  "<literalchar data='&#60;'>"    
       -- ``less than'' sign--> 
<!ENTITY  greaterthan  "<literalchar data='>'>"    
       -- ``greater than'' sign--> 
<!ENTITY  ampersand "<literalchar data='&#38;'>"    
       -- ``ampersand'' sign--> 
@} 
 
\chapter{SGML Tangle}
\label{sec:tangle}


\section{Purpose}

The \app{SGMLTangle} style sheet performs the ``tangle'' phase of
literate programming. In other words, it takes the literate
programming code and rearranges it so that it is acceptable to the
computer as a computer program (assuming, of course, that the
programmer has correctly written the program!)

\section{Implementation}

In order to make it as simple as possible to bring this system up on
a new machine, this style sheet is presented as a single scrap.  Notes
on the program follow the implementation scrap.

@O SGMLTangle.dsl @{
<!DOCTYPE style-sheet  
  PUBLIC "-//James Clark//DTD DSSSL Style Sheet//EN"> 
<style-sheet> 
<style-specification 
 id = "tangle"> 
<style-specification-body>

 (declare-flow-object-class entity 
   "UNREGISTERED::James Clark//Flow Object Class::entity") 
 (declare-flow-object-class formatting-instruction 
   "UNREGISTERED::James Clark//Flow Object Class::formatting-instruction")

 (default (process-node-list (element-children)))

 (element programlisting
  (make sequence 
   (if (attribute-string "file") 
     (make entity 
       system-id: (attribute-string "file") 
       (make sequence 
         (process-children) 
         (if (attribute-string "continuedin")
           (with-mode continuation
             (process-element-with-id 
               (attribute-string "continuedin")))
           (empty-sosofo))))
      (empty-sosofo))))
  
 (element (programlisting xref)
   (with-mode definition
     (process-element-with-id 
       (attribute-string "linkend"))))

 (mode definition
   (element programlisting
     (make sequence
       (process-children)
       (if (attribute-string "continuedin")
           (with-mode continuation
             (process-element-with-id 
               (attribute-string "continuedin")))
           (empty-sosofo)))))
  
 (mode continuation
   (element programlisting
     (make sequence
       (process-children)
       (if (attribute-string "continuedin")
           (with-mode continuation
             (process-element-with-id 
               (attribute-string "continuedin")))
           (empty-sosofo)))))
     
 (element literalchar 
  (make sequence 
   (make formatting-instruction 
     data: (attribute-string "data")))) 

 @<Define element-children function@>

</style-specification-body>
</style-specification> 
</style-sheet> 
@}

\texttt{(element-children snl)} finds all of the elements that are
direct children of singleton nodelist ``snl''.  This is useful for
filtering, processing, and counting, when you're not interested in any
text, PI, or comment nodes.  This function (and the default processing
rule) were provided by Christopher R. Maden \texttt{<crism@@maden.org>}
in a message on the DSSSList dated Mon, 02 Apr 2001 22:36:33 -0700, and
titled ``Re: (dsssl) "Default" processing rule?''.

@D Define element-children function @{
 (define (element-children #!optional (snl (current-node)))
   (select-by-class (children snl)
                    'element))
@}


\section{Implementation Notes}


\chapter{SGML Weave}
\label{sec:weave}

\section{Purpose}

The ``SGMLWeave'' program (\texttt{SGMLWeave.dsl}) is a relatively
minor customization of Norman Walsh's \ac{DSSSL} stylesheets for
\app{DocBook}.  In fact, for minimum functionality, the \emph{only}
necessary change to the stylesheets is the addition of a processing
rule for the \tag{literalchar} element, and that processing rule
(shown in Section~\ref{sec:literalchar-rule} is relatively simple.

This is in fact a design goal of the DocBook-based literate
programming system, as it exploits the significant efforts of a number
of people to develop tools for \app{DocBook}.

\section{Minimum DocBook Customization Layer}

@O SGMLWeave.dsl @{<!--
PUBLIC "-//Mark Wroth//DOCUMENT DBLP Weave Print Rules 1.0//EN"

This document is intended to be a minimum DocBook style-sheet
customization layer.
-->
<!DOCTYPE style-sheet
  PUBLIC "-//James Clark//DTD DSSSL Style Sheet//EN" [
 <!ENTITY dbprint.dsl PUBLIC 
   "-//Norman Walsh//DOCUMENT DocBook Print Stylesheet//EN"
   CDATA DSSSL>
 <!ENTITY dbhtml.dsl PUBLIC
   "-//Norman Walsh//DOCUMENT DocBook HTML Stylesheet//EN"
   CDATA DSSSL>
]> 
<style-sheet> 
<style-specification id="print" use="dbprint"> 
<style-specification-body>

@<literalchar print processing rule@>
@<programlisting print customizations@>

</style-specification-body>
</style-specification>
<style-specification id="html" use="dbhtml"> 
<style-specification-body>

@<literalchar print processing rule@>
@<programlisting HTML customizations@>

</style-specification-body>
</style-specification>
<external-specification id="dbprint" document="dbprint.dsl">
<external-specification id="dbhtml" document="dbhtml.dsl">
</style-sheet>
@}

\section{The literalchar Processing Rules}
\label{sec:literalchar-rule}

@D literalchar print processing rule @{(element literalchar
  (make sequence
    (literal (attribute-string "data"))))
@}


\section{The programlisting Customizations}
\label{sec:pl-rules}

The basis of our customization is the existing \tag{programlisting}
code from \cite[dbverb.dsl]{walsh-dsssl}.  We will modify this by
adding information ahead of the basic element (by identifying the
scrap by either the file name or the definition name, depending on the
type of scrap), and adding information after the program listing (if
the scrap is continued).


\subsection{Print Customizations}

Almost everything about the print stylesheets will remain untouched.
However, there are a few changes that need to be made to the
processing of the \texttt{printlisting} element.  The
\texttt{printlisting} element itself gets some additional information
added to it based on its place in the literate programming web, and
the special use of \tag{xref} means that we should wrap the cross
reference text in some punctuation to indicate that it is a code
section reference rather than actual code.
@D programlisting print customizations @{
@<Customize the programlisting proper@>
@<Customize the (programlisting xref)@>
@<Print auxiliary definitions@>
@}


\subsubsection{The printlisting Proper}

There are essentially two sets of changes we want to make to the
processing of the \texttt{programlisting} element: adding header
information identifying the code section and the particular scrap, and
adding continuation information if the scrap has a continuation.

Mechanically, we do this by wrapping the original code (taken from
\cite[\texttt{dbverb.dsl}]{walsh-dsssl}) inside a \texttt{make
  sequence} and adding the code to produce the header and continuation
text before and after the original code.

@D Customize the programlisting proper @{
(element programlisting (make sequence
  @<Provide scrap header information@>
  @<Original programlisting print code@>
  @<Provide optional continuation info@> ))
@}

The scrap header information is straight forward except for the actual
title of the code section.  That title is either the file name of the
disk file the section will be written to, or the \texttt{xreflabel} of
the section.  In either case, the title is found in the first scrap of
the section, which may or may not be the scrap we are currently
processing. 

We deal with this as a set of nested \texttt{if}
statements to grab the title if we are in the first scrap of the
section, or processing the \texttt{continuedfrom} scrap with a special
mode that recursively hunts back up the linked list until it gets to
the first scrap.\footnote{It is exactly this application that caused
  us to define the programlisting scraps as a doubly-linked list.}

The other refinement is that we will set the header and continuation
information in a specific font and size intended to be distinct
from the general font and size used in the document.  For ease of
reference, these are defined in the auxiliary information.

@D Provide scrap header information @{
(make paragraph
  (make sequence
    font-family-name: scrap-header-font
    font-size:        scrap-header-size
    (literal "\left-pointing-angle-bracket")
    (if (attribute-string "file")
      (literal (attribute-string "file"))
      (if (attribute-string "xreflabel")
        (literal (attribute-string "xreflabel"))
        (with-mode scrap-title-mode
          (make sequence
            (process-element-with-id 
              (attribute-string "continuedfrom"))
            (literal " ")))))
     (make sequence
       font-size: (* scrap-header-size 0.75)
       (literal " (ID: ")
       (literal (attribute-string "id")))
     (literal ")\right-pointing-angle-bracket")
     (if (attribute-string "continuedfrom")
         (literal "+")
         (empty-sosofo))
     (literal "\identical-to")))
@}

@D Provide optional continuation info @{
(if (attribute-string "continuedin")
    (make paragraph
      font-family-name: scrap-header-font
      font-size:        scrap-header-size
      (make sequence
        (literal "Continued in ")
        (literal (attribute-string "continuedin"))))
    (empty-sosofo))
@}


\subsubsection{The printlisting xref Element}

The main customization we want for the \texttt{xref} element is to put
the cross-reference text in a running text font, and enclose it in
angle brackets.  In both cases, this is to distinguish a code section
reference from actual code.  

For consistency, we will put the reference in the same font and size
as the scrap header and continuation information.
@D Customize the (programlisting xref) @{
(element (programlisting xref) (make sequence
  font-family-name: scrap-header-font
  font-size:        scrap-header-size
 (literal "\left-pointing-angle-bracket")
 @<Original xref print code@>
 (literal "\right-pointing-angle-bracket")
))
@}


\subsubsection{Print Auxiliary Definitions}

Here we define the common information such as the size and font used
for the header and continuation.  Additionally, we define the special
mode used to find the code section title here.

@D Print auxiliary definitions @{
(define scrap-header-size 8pt)
(define scrap-header-font "Georgia")
@<Define scrap-title-mode@>
@}

The \texttt{scrap-title-mode} is defined to process
\texttt{programlisting} elements and either extract the title (found
in the \texttt{file} or \texttt{xreflabel} attributes, or to continue
up the linked list if neither attribute is present.
@D Define scrap-title-mode @{
(mode scrap-title-mode
  (element programlisting
    (make sequence
      (if (attribute-string "file")
          (literal (attribute-string "file"))
          (empty-sosofo))
      (if (attribute-string "xreflabel")
          (literal (attribute-string "xreflabel"))
          (empty-sosofo)))))
@}

\subsubsection{Original Modular Style Sheet Print Code}

This is the complete \ac{DSSSL} code for the \texttt{programlisting}
element, found in \texttt{dbverb.dsl}.
@D Original programlisting print code @{
($verbatim-display$
 %indent-programlisting-lines%
 %number-programlisting-lines%)@}

This is the complete \ac{DSSSL} code found in \texttt{dblink.dsl} for
the \texttt{xref} element.  It is probably \emph{much} more complex
than is actually needed for the rather specialized use we are making
of it, but it is easier to just reproduce it than to try to simplify
it.

@D Original xref print code @{
 (let* ((endterm (attribute-string (normalize "endterm")))
         (linkend (attribute-string (normalize "linkend")))
         (target  (element-with-id linkend))
         (xreflabel (if (node-list-empty? target)
                        #f
                        (attribute-string (normalize "xreflabel") target))))
    (if (node-list-empty? target)
        (error (string-append "XRef LinkEnd to missing ID '" linkend "'"))
        (if xreflabel
            (make link 
              destination: (node-list-address target)
              (literal xreflabel))
            (if endterm
                (if (node-list-empty? (element-with-id endterm))
                    (error (string-append "XRef EndTerm to missing ID '" 
                                          endterm "'"))
                    (make link 
                      destination: (node-list-address (element-with-id endterm))
                      (with-mode xref-endterm-mode 
                        (process-element-with-id endterm))))
                (cond
                 ((or (equal? (gi target) (normalize "biblioentry"))
                      (equal? (gi target) (normalize "bibliomixed")))
                  ;; xref to the bibliography is a special case
                  (xref-biblioentry target))
                 ((equal? (gi target) (normalize "co"))
                  ;; callouts are a special case
                  (xref-callout target))
                 ((equal? (gi target) (normalize "listitem"))
                  (xref-listitem target))
                 ((equal? (gi target) (normalize "question"))
                  (xref-question target))
                 ((equal? (gi target) (normalize "answer"))
                  (xref-answer target))
                 ((equal? (gi target) (normalize "refentry"))
                  (xref-refentry target))
                 ((equal? (gi target) (normalize "glossentry"))
                  ;; as are glossentrys
                  (xref-glossentry target))
                 ((equal? (gi target) (normalize "author"))
                  ;; and authors
                  (xref-author target))
                 ((equal? (gi target) (normalize "authorgroup"))
                  ;; and authorgroups
                  (xref-authorgroup target))
                 (else 
                  (xref-general target)))))))@}


\subsection{HTML Customizations}

In the \ac{HTML} style sheet, we need to make the same kinds of
customization we did with the print style sheet.  The details of the
actual customizations differ slightly.
@D programlisting HTML customizations @{
@<Programlisting element HTML customization@>
@<xref element HTML customization@>
@<HTML auxiliary definitions@>
@}

\subsubsection{The programlisting Proper}

In the \texttt{programlisting}, we make the same kinds of additions
(header and continuation information) as used in the print style sheet.

@D Programlisting element HTML customization @{
(element programlisting (make sequence
 @<Provide HTML scrap header information@>
 @<Original programlisting HTML code@>
 @<Provide optional HTML scrap continuation info@>
))@}

The code is also largely common, with some minor changes for the
\ac{HTML} output and its limited character repertoire.
@D Provide HTML scrap header information @{
(make element gi: "P"
  (make sequence
    (literal "<")
    (if (attribute-string "file")
      (literal (attribute-string "file"))
      (if (attribute-string "xreflabel")
        (literal (attribute-string "xreflabel"))
        (with-mode scrap-title-mode
          (make sequence
            (process-element-with-id 
              (attribute-string "continuedfrom"))
            (literal " ")))))
     (make sequence
       (literal " (ID: ")
       (literal (attribute-string "id")))
     (literal ")>")
     (if (attribute-string "continuedfrom")
         (literal "+")
         (empty-sosofo))
     (literal "=")))@}

@D Provide optional HTML scrap continuation info @{
(if (attribute-string "continuedin")
    (make element gi: "P"
      (make sequence
        (literal "Continued in ")
        (literal (attribute-string "continuedin"))))
    (empty-sosofo))@}

\subsubsection{Customizing the xref Element}

@D xref element HTML customization @{
(element (programlisting xref) (make sequence
(literal "<")
@<Original xref HTML code@>
(literal ">")
))@}


\subsubsection{HTML Auxiliary Definitions}

Here we employ the same basic structure as with the print style sheet,
although there is less to define; this primarily is intended to allow
room for future elaboration of the style sheet.

The \ac{DSSSL} code for the \texttt{scrap-title-mode} is identical
with that used in the print style sheet, so we simply re-use it.
@D HTML auxiliary definitions @{
@<Define scrap-title-mode@>
@}

\subsubsection{Original programlisting Code}

@D Original programlisting HTML code @{
($verbatim-display$
%indent-programlisting-lines%
 %number-programlisting-lines%)
@}

\subsubsection{Original programlisting Code}

@D Original xref HTML code @{
(let* ((endterm   (attribute-string (normalize "endterm")))
         (linkend   (attribute-string (normalize "linkend")))
         (target    (element-with-id linkend))
         (xreflabel (if (node-list-empty? target)
                        #f
                        (attribute-string (normalize "xreflabel") target))))
    (if (node-list-empty? target)
        (error (string-append "XRef LinkEnd to missing ID '" linkend "'"))
        (make element gi: "A"
              attributes: (list
                           (list "HREF" (href-to target)))
              (if xreflabel
                  (literal xreflabel)
                  (if endterm
                      (if (node-list-empty? (element-with-id endterm))
                          (error (string-append
                                  "XRef EndTerm to missing ID '" 
                                  endterm "'"))
                          (with-mode xref-endterm-mode
                            (process-node-list (element-with-id endterm))))
                      (cond
                       ((or (equal? (gi target) (normalize "biblioentry"))
                            (equal? (gi target) (normalize "bibliomixed")))
                        ;; xref to the bibliography is a special case
                        (xref-biblioentry target))
                       ((equal? (gi target) (normalize "co"))
                        ;; callouts are a special case
                        ($callout-mark$ target #f))
                       ((equal? (gi target) (normalize "listitem"))
                        ;; listitems are a special case
                        (if (equal? (gi (parent target)) (normalize "orderedlist"))
                            (literal (orderedlist-listitem-label-recursive target))
                            (error (string-append "XRef to LISTITEM only supported in ORDEREDLISTs"))))
                       ((equal? (gi target) (normalize "question"))
                        ;; questions and answers are (yet another) special case
                        (make sequence
                          (literal (gentext-element-name target))
                          (literal (gentext-label-title-sep target))
                          (literal (question-answer-label target))))
                       ((equal? (gi target) (normalize "answer"))
                        ;; questions and answers are (yet another) special case
                        (make sequence
                          (literal (gentext-element-name target))
                          (literal (gentext-label-title-sep target))
                          (literal (question-answer-label target))))
                       ((equal? (gi target) (normalize "refentry"))
                        ;; so are refentrys
                        (xref-refentry target))
                       ((equal? (gi target) (normalize "glossentry"))
                        ;; as are glossentrys
                        (xref-glossentry target))
                       ((equal? (gi target) (normalize "author"))
                        ;; and authors
                        (xref-author target))
                       ((equal? (gi target) (normalize "authorgroup"))
                        ;; and authorgroups
                        (xref-authorgroup target))
; this doesn't really work very well yet
;                      ((equal? (gi target) (normalize "substeps"))
;                       ;; and substeps
;                       (xref-substeps target))
                       (else 
                        (xref-general target))))))))@}
\chapter{Sample Literate Program}

\section{The Sample Document} 
@O sample.sgm @{<!DOCTYPE article 
 PUBLIC "-//Mark Wroth//DTD DocBook V4.1-Based Extension Literate Programming 1.0//EN">
<article id="sample-lp">
  <title>A Sample DocBook-Based Literate Program</title>
  <section>
    <title>Introduction</title>
    <para>This is a sample document illustrating basic use of the
    DocBook-based literate programming tool.</para>
    <para>This document combines human-readable documentation of 
    a computer program with the actual computer-readable source 
    code.  Depending on how it is processed, it becomes either 
    printed (on-line) documentation of the program, or the actual 
    source submitted to the computer for compilation (or
    interpretation, depending on the language).</para>
  </section>
  <section>
   <title>Source Code</title>

   <para>The source code is divided into a number of
   <quote>scraps</quote>, each containing a discrete fragment of
   code.  These scraps are assembled into code sections by
   concatenating the header scrap with the various continuation
   scraps, in an order defined by the programmer. File sections are
   written to the file indicated by the programmer, while definition
   sections are inserted at a place or places defined by the
   programmer. </para>
   
   <para>The first code scrap defines a file output, specifically 
   to <filename>sample.code</filename>.</para>

   <programlisting
     id="scrap1"
     file="sample.code"
     continuedin="scrap2"
     >
-- This is sample code in an imaginary language
-- Taken from the first scrap
if a &lessthan; b then
  <xref linkend="scrap3">
fi
   </programlisting>

   <para>The next code scrap is a continuation of the first
   scrap.</para>   

   <programlisting
    id="scrap2"
    continuedfrom="scrap1"
    >
-- This is continued code, taken from the second scrap
--
set c = a &ampersand; b 
 greater than: &greaterthan;
   </programlisting>
 
   <para>The following code section is an example of a definition
   scrap, and will be included in a file output scrap.</para>
   <programlisting
    id="scrap3"
    xreflabel="The Third Scrap"
    continuedin="scrap4">
-- Yet more program code from the third scrap
   </programlisting>
   
   <para>Finally, we have a continuation scrap continuing
   a definition scrap.</para>

   <programlisting
     id="scrap4"
     continuedfrom="scrap3"
   >
-- This is scrap 4, which continues scrap 3
-- It should appear where scrap 3 was inserted.
   </programlisting>
 </section>
</article>
@}


\chapter{System Performance}
\label{sec:testing}

Generally speaking, verifying the functionality of the literate
programming system consists of checking that the \app{tangle} branch
correctly produces the intended code, and the \app{weave} branch
produces appropriate human-readable documentation.

\section{Sample Code Output}

The code output file \texttt{sample.code} produced from the sample
document looks like this: 

\verbatiminput{sample.code}

This demonstrates the key functions needed in the \app{tangle} branch,
namely that:
\begin{itemize}
\item Code sections are correctly assembled from the separate scraps
  identified in the source code.
\item Definition sections are inserted into code sections at the
  location identified by the \tag{xref} tag pointing to the head of
  the definition section.
\item File output sections are written to disk in with the desired
  file names. 
\item Syntactically significant characters (to \ac{SGML}) including
  \verb|<|, \verb|>|, and \verb|&|, are written correctly in the
  output file.
\end{itemize}

The handling of whitespace in the code scraps is not quite what I
expected, but appears to be consistent and reasonable.  It appears
that whitespace is (correctly) transcribed to the output file, except
that an \ac{SGML} record end (\texttt{CR-LF} pair, under
\app{Windows}) following the \tag{programlisting} start tag is
\emph{not} transcribed to the output.


\section{Sample Woven Output}

The \app{weave} \ac{DSSSL} style sheets produce two sets of
human-readable documentation from \file{sample.sgm}: \file{sample.rtf}
and a set of \ac{HTML} files.  The \ac{HTML} files are divided and
named by the conventions of the \ac{HTML} \textsl{Modular DocBook
  Style Sheets}, which (without customization) produce multiple small
files.

In both cases, the \app{weave} outputs reasonably produce verbatim
program listings, with header and continuation information that
matches the actual structure defined by the sample file.

The typographic treatment of the header and continuation is
acceptable, although the amount of vertical space introduced in both
the print and \ac{HTML} versions between the header and the body of
the programlisting is larger than I would prefer.


\section{Evaluation}
\label{sec:testeval}

Overall, these implementations of a \ac{DTD}, \app{tangle}, and
\app{weave} are functional.  They do not produce really high-quality
typographical output, but they do reasonably display the structure of
the code sections.  Pending further experience with the system, this
seems to be an acceptable implementation.

\appendix

\chapter*{Acronyms}
\label{sec:acronyms}

\begin{acronym}
  \acro{DSSSL}{Document Style Semantics Specification Language}{ DSSSL
  is an ISO standard defining how to specify transformations from
  \ac{SGML} documents to page-oriented output renderings.}
  \acro{DTD}{Document Type Definition, Document Type Declaration}
  \acro{HTML}{Hypertext Markup Language}
  \acro{ISO}{International Organization for Standardization}
  \acro{OASIS}{Organization for the Advancement of Structured Information Standards}
  \acro{OSNL}{Optional Singleton Node List}
  \acro{SGML}{Standard Generalized Markup Language}
  \acro{SNL}{Singleton Node List}
  \acro{XML}{Extensible Markup Language}
\end{acronym}

\begin{thebibliography}{99}
\bibitem{Carlisle3-241} Carlisle, David.  ``Re: Issues with literate
  programming DSSSL Script'', in \textsl{DSSSList Digest} Vol~3,
  Number~241 (Thu, 16 Dec 1999 16:27:27 GMT).
  \texttt{<davidc@@nag.co.uk>}.  
  
\bibitem{Knuth92} Knuth, Donald. \textsl{Literate Programming}.
  Stanford, CA: Center for the Study of Language and Information,
  1992.

\bibitem{Maden01-04-02} Maden, Christopher R.. ``Re: (dsssl) "Default"
  processing rule?'', in \textsl{DSSSList Digest} Vol~4, Number~143
  (Mon, 02 Apr 2001 22:36:33 (-0700)). \texttt{<crism@@maden.org>}.
  
\bibitem{docbook4.1} \ac{OASIS}. \textsl{DocBook}.
  \url{http://www.oasis-open.org/docbook}.

\bibitem{walsh-dsssl} Norman Walsh. \textsl{The Modular DocBook
    Stylesheets}. \url{http://www.nwalsh.com/docbook/dsssl/index.html}.
\end{thebibliography}

%\chapter*{Indicies}


\end{document}
